import { Router } from "express";
import {
  ApplicationType,
  TypedRequest,
  AnalyseApplicationPayload,
} from "../config/types";
import { authMiddleware, candidateMiddleware } from "../middlewares/auth";
import Application from "../models/Application";
import Jobs from "../models/Job";
import ResumeAPI from "../utilities/apis/resume";
import VoiceAPI from "../utilities/apis/voice";

const router = Router();

router.post(
  "/apply",
  authMiddleware,
  candidateMiddleware,
  async (
    req: TypedRequest<
      { userId: string },
      { application: ApplicationType; resumeUrl: string }
    >,
    res
  ) => {
    try {
      const { application, resumeUrl } = req.body;
      const job = await Jobs.findById(application.job);

      const data: any = await ResumeAPI.getResumeScores({
        user_id: req.query.userId,
        resume_url: resumeUrl,
        primary_skills: job.primarySkills,
        secondary_skills: job.secondarySkills,
        job_desc: job.description,
      });

      const score: ApplicationType["score"] = {
        primary: data.primary_score,
        primatyMatch: data.primary_match,
        secondary: data.secondary_score,
        secondaryMatch: data.secondary_match,
        similarity: data.similarity,
        total: data.primary_score + data.secondary_score + data.similarity,
      };

      const newApplication = new Application({ ...application, score });
      const _application = await newApplication.save();
      job.applications.push(_application.id);
      await job.save();
      return res.json({
        success: true,
        applicationId: _application.id,
      });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

router.put(
  "/update",
  authMiddleware,
  async (
    req: TypedRequest<
      {},
      {
        applicationId: string;
        update: Partial<ApplicationType>;
        candidateId: string;
      }
    >,
    res
  ) => {
    let update = req.body.update;

    if (update.interview?.videoRef) {
      const score: any = await VoiceAPI.verifyVoice({
        video_url: update.interview?.videoRef,
        user_id: req.body.candidateId,
        application_id: req.body.applicationId,
      });

      update.interview.voiceVerification = score;
    }

    Application.findByIdAndUpdate(req.body.applicationId, {
      $set: update,
    })
      .then((_application) => {
        res.json({
          success: true,
          application: { ..._application, ...update },
        });
      })
      .catch((err) => res.send(err));
  }
);

router.post(
  "/analyse",
  authMiddleware,
  async (req: TypedRequest<{}, AnalyseApplicationPayload>, res) => {
    const { applicationId, startTime, endTime } = req.body;
    const data = await VoiceAPI.analyseVoice({
      start: startTime,
      end: endTime,
      application_id: applicationId,
    });

    return res.json({ voice: data });
  }
);

module.exports = router;
