import { Router } from "express";
import {
  ApplicationType,
  JobType,
  TypedRequest,
  USER_TYPE,
} from "../config/types";
import {
  authMiddleware,
  candidateMiddleware,
  organizationMiddleware,
} from "../middlewares/auth";
import Application from "../models/Application";
import Auth from "../models/Auth";
import Jobs from "../models/Job";
import ResumeAPI from "../utilities/apis/resume";

const router = Router();

router.get(
  "/",
  authMiddleware,
  async (req: TypedRequest<{ userId: string }, null>, res) => {
    try {
      const user = await Auth.findOne({ userId: req.query.userId });

      let jobs;

      if (user.userType === USER_TYPE.CANDIDATE) {
        jobs = await Jobs.find()
          .populate({
            path: "applications",
            select: ["candidate", "status"],
          })
          .populate({ path: "organization" });
      } else {
        jobs = await Jobs.find({ organization: req.query.userId }).populate({
          path: "applications",
          populate: {
            path: "candidate",
            select: [
              "name",
              "contacts",
              "dateOfBirth",
              "profilePicture",
              "resume",
              "resumeData",
              "selfIntro",
            ],
          },
        });
      }

      return res.json({ jobs, success: true });
    } catch (error) {
      return res.json({ error, success: false });
    }
  }
);

router.post(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, JobType>, res) => {
    try {
      const newJob = new Jobs({ ...req.body, organization: req.query.userId });
      const job = await newJob.save();
      return res.json({ success: true, job });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

router.put(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, JobType>, res) => {
    try {
      const job = await Jobs.findByIdAndUpdate(req.body._id, {
        $set: req.body,
      });
      return res.json({ success: true, job });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

router.delete(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, { jobId: string }>, res) => {
    try {
      await Jobs.deleteOne({
        organization: req.query.userId,
        _id: req.body.jobId,
      });
      return res.json({ success: true });
    } catch (error) {
      return res.json({ success: false });
    }
  }
);

module.exports = router;
